@@ -43,17 +43,15 @@ def get_coverage_file():
 
 
 def test_null(update_date=None):
-    return core.db.select(Application, optional_filters=[Application.update_date == update_date])
+    return core.db.select(
+        Application, optional_filters=[Application.update_date == update_date]
+    )
 
 
 @route(
     "test_null",
     parameters=[
-        Parameter(
-            "update_date",
-            required=False,
-            none_value=sqlalchemy.null(),
-        )
+        Parameter("update_date", required=False, none_value=sqlalchemy.null(),)
     ],
 )
 def test():
