@@ -12,11 +12,11 @@ UNIX_LINE_ENDING = "\n"
 
 def standardize_content(content: str):
     if type(content) != str:
-        content = content.decode('utf-8')
+        content = content.decode("utf-8")
     return content.replace("\\r\\n", "\n").replace("\\r\n", "\n").replace("\\t", "\t")
 
 
-def process_content(content: str, parameters:Dict[str,object]=None):
+def process_content(content: str, parameters: Dict[str, object] = None):
     if content.startswith("file:"):
         path = content.replace("file:", "")
         if not os.path.exists(path):
@@ -25,7 +25,7 @@ def process_content(content: str, parameters:Dict[str,object]=None):
             content = f.read()
     if parameters is not None:
         for key, value in parameters.items():
-            content = content.replace("{{%s}}"%key, value)
+            content = content.replace("{{%s}}" % key, value)
     return content
 
 
@@ -162,14 +162,17 @@ class AlphaSsh:
         output = self.execute_cmd("test -f %s && echo 'y'" % filename)
         return "y" in output
 
+    def is_directory(self, path: str, group: str = None, user: str = None, mode=None):
+        return self.is_dir(path=path, group=group, user=user, mode=mode)
+
     def is_dir(self, path: str, group: str = None, user: str = None, mode=None):
         output = self.execute_cmd("test -d %s && echo 'y'" % path)
         if group is not None and not self.is_group(group, path):
-            self.change_group(group, path)
+            return False
         if user is not None and not self.is_user(user, path):
-            self.change_user(user, path)
+            return False
         if mode is not None and not self.is_mode(mode, path):
-            self.change_mode(mode, path)
+            return False
         return "y" in output
 
     def make_directory(
@@ -185,8 +188,8 @@ class AlphaSsh:
             options = "-p"
         if not self.is_dir(path):
             self.execute_cmd("mkdir %s %s" % (options, path))
-        if not self.is_dir(path):
-            return False
+            if not self.is_dir(path):
+                return False
         if group is not None:
             self.change_group(group, path)
         if user is not None:
@@ -301,11 +304,17 @@ class AlphaSsh:
         self.execute_cmd(f"touch {path}")
         return self.is_file(path)
 
-    def write_to_file(self, content: str, path: str, ensure_path: bool = True, parameters:Dict[str, object]=None) -> bool:
+    def write_to_file(
+        self,
+        content: str,
+        path: str,
+        ensure_path: bool = True,
+        parameters: Dict[str, object] = None,
+    ) -> bool:
         if ensure_path:
             self.make_directory(os.path.dirname(path))
 
-        if content.startswith("file:"): #TODO: remove
+        if content.startswith("file:"):  # TODO: remove
             content_path = content.replace("file:", "")
             self.scp.put(content_path, path)
             return True
