@@ -4,6 +4,7 @@ from logging.handlers import TimedRotatingFileHandler
 import platform
 
 from numpy.core.numeric import full
+import unidecode
 
 PLATFORM = platform.system().lower()
 
@@ -16,8 +17,10 @@ TIMINGS = []
 
 base_time = datetime.datetime.now()
 
-DEFAULT_FORMAT = "{$date} - {$level:7} - {$pid:5} - {$file:>15}.{$line:<4} - {$name:<14}: $message" # %(processName)s %(filename)s:%(lineno)s
+DEFAULT_FORMAT = "{$date} - {$level:7} - {$pid:5} - {$file:>15}.{$line:<4} - {$name:<14}: $message"  # %(processName)s %(filename)s:%(lineno)s
 DEFAUT_DATE_FORMAT = "%Y-%m-%d %H:%M:%S"
+
+
 class NoParsingFilter(logging.Filter):
     def __init__(self, name="", excludes={}, level=None):
         super().__init__(name)
@@ -34,13 +37,25 @@ class NoParsingFilter(logging.Filter):
                         return False
         return True
 
+
 class AlphaLogger:
     error_logger = None
     monitoring_logger = None
 
     def to_json(self):
-        keys = ["level", "database_name", "excludes", "date_format","format_log", "date_str","root","pid","name","monitor"]
-        return {x:self.__dict__[x] for x in keys if x in self.__dict__}
+        keys = [
+            "level",
+            "database_name",
+            "excludes",
+            "date_format",
+            "format_log",
+            "date_str",
+            "root",
+            "pid",
+            "name",
+            "monitor",
+        ]
+        return {x: self.__dict__[x] for x in keys if x in self.__dict__}
 
     def __init__(
         self,
@@ -49,17 +64,16 @@ class AlphaLogger:
         root: str = None,
         cmd_output: bool = True,
         level: str = "INFO",
-        format_log:str = DEFAULT_FORMAT,
-        date_format:str = DEFAUT_DATE_FORMAT,
+        format_log: str = DEFAULT_FORMAT,
+        date_format: str = DEFAUT_DATE_FORMAT,
         colors=None,
-        database=None,
+        log_in_db: bool = False,
         excludes=None,
         replaces=None,
-        config={}
+        config={},
     ):
         self.date_str: str = ""
-        self.database_name: str = database
-        self.database = None
+        self.log_in_db = None
         self.excludes = excludes
         self.replaces = replaces
         self.config = config
@@ -88,20 +102,33 @@ class AlphaLogger:
         # File handler
         if config is not None and "timed" in config:
             if config is not None and len(config) != 0:
-                handler = TimedRotatingFileHandler(
-                    log_path, **config
-                )
+                handler = TimedRotatingFileHandler(log_path, **config)
             else:
                 handler = TimedRotatingFileHandler(
                     log_path, when="midnight", interval=1, backupCount=90
                 )
         else:
-            defaults = {"mode":"a","maxBytes":512 * 1024,"backupCount":10,
-            "encoding":None, "debug":None, "delay":None, "use_gzip":None,
-            "owner":None, "chmod":None, "umask":None, "newline":None, "terminator":None,
-            "unicode_error_policy":None}
-            config = {} if config is None else {x:y for x,y in config.items() if x in defaults}
-            for x,y in defaults.items():
+            defaults = {
+                "mode": "a",
+                "maxBytes": 10 * 1024 * 1024,
+                "backupCount": 10,
+                "encoding": None,
+                "debug": None,
+                "delay": None,
+                "use_gzip": None,
+                "owner": None,
+                "chmod": None,
+                "umask": None,
+                "newline": None,
+                "terminator": None,
+                "unicode_error_policy": None,
+            }
+            config = (
+                {}
+                if config is None
+                else {x: y for x, y in config.items() if x in defaults}
+            )
+            for x, y in defaults.items():
                 if not x in config and y is not None:
                     config[x] = y
             handler = ConcurrentRotatingFileHandler(log_path, **config)
@@ -135,8 +162,9 @@ class AlphaLogger:
         stack_level: int,
         level: str = "INFO",
         monitor: str = None,
-        save=False,
+        log_in_db: bool = False,
         ex: Exception = None,
+        enabled: bool = True,
     ):
         """
                 frame       = inspect.stack()[1]
@@ -145,10 +173,12 @@ class AlphaLogger:
         if module is not None:
             origin  = os.path.basename(module.__file__)
         """
+        if not enabled:
+            return
         if message is None and ex is not None:
             message = "Exception"
         if monitor:
-            save = True
+            log_in_db = True
 
         if isinstance(message, Exception):
             message = traceback.format_exc()
@@ -159,27 +189,31 @@ class AlphaLogger:
 
         if self.replaces is not None and type(self.replaces) == dict:
             for regex, replacement in self.replaces.items():
-                matchs = re.findall(regex,full_message)
+                matchs = re.findall(regex, full_message)
                 if len(matchs) != 0:
                     for match in matchs:
                         full_message = full_message.replace(match, replacement)
 
         if ex is not None:
-            full_message += str(ex) + "\n" + traceback.format_exc()
+            full_message += " :" + str(ex) + "\n" + traceback.format_exc()
 
         fct = getattr(self.logger, level.lower())
         fct(full_message)
 
         if monitor is not None and self.monitoring_logger is not None:
             fct_monitor = getattr(self.monitoring_logger, level.lower())
-            fct_monitor(
-                message=full_message.replace(message, f"[{monitor}] {message}")
-            )
+            msg = full_message.replace(message, f"[{monitor}] {message}")
+            msg = unidecode.unidecode(msg)
+            fct_monitor(message=msg)
 
         self.last_level = level.upper()
         self.last_message = message
 
-        if level.lower() in ["critical", "error", "warning"] and self.error_logger is not None and self.name != "error":
+        if (
+            level.lower() in ["critical", "error", "warning"]
+            and self.error_logger is not None
+            and self.name != "error"
+        ):
             fct_errors = getattr(self.error_logger, level.lower())
             fct_errors(full_message)
 
@@ -190,7 +224,7 @@ class AlphaLogger:
         })
         base_time = datetime.datetime.now()"""
 
-        if save:
+        if log_in_db:
             type_ = "exception" if ex is not None else "message"
             self.__log_in_db(full_message, origin="alpha", level=level, type_=type_)
 
@@ -233,7 +267,13 @@ class AlphaLogger:
         return msg
 
     def info(
-        self, message=None, monitor=None, level=1, save=False, ex: Exception = None
+        self,
+        message=None,
+        monitor=None,
+        level=1,
+        log_in_db=False,
+        ex: Exception = None,
+        enabled: bool = True,
     ):
         self._log(
             message,
@@ -241,12 +281,19 @@ class AlphaLogger:
             stack_level=level,
             level="info",
             monitor=monitor,
-            save=save,
+            log_in_db=log_in_db,
             ex=ex,
+            enabled=enabled,
         )
 
     def warning(
-        self, message=None, monitor=None, level=1, save=False, ex: Exception = None
+        self,
+        message=None,
+        monitor=None,
+        level=1,
+        log_in_db=False,
+        ex: Exception = None,
+        enabled: bool = True,
     ):
         self._log(
             message,
@@ -254,12 +301,19 @@ class AlphaLogger:
             stack_level=level,
             level="warning",
             monitor=monitor,
-            save=save,
+            log_in_db=log_in_db,
             ex=ex,
+            enabled=enabled,
         )
 
     def error(
-        self, message=None, monitor=None, level=1, save=False, ex: Exception = None
+        self,
+        message=None,
+        monitor=None,
+        level=1,
+        log_in_db=False,
+        ex: Exception = None,
+        enabled: bool = True,
     ):
         self._log(
             message,
@@ -267,12 +321,19 @@ class AlphaLogger:
             stack_level=level,
             level="error",
             monitor=monitor,
-            save=save,
+            log_in_db=log_in_db,
             ex=ex,
+            enabled=enabled,
         )
 
     def debug(
-        self, message=None, monitor=None, level=1, save=False, ex: Exception = None
+        self,
+        message=None,
+        monitor=None,
+        level=1,
+        log_in_db=False,
+        ex: Exception = None,
+        enabled: bool = True,
     ):
         self._log(
             message,
@@ -280,12 +341,19 @@ class AlphaLogger:
             stack_level=level,
             level="debug",
             monitor=monitor,
-            save=save,
+            log_in_db=log_in_db,
             ex=ex,
+            enabled=enabled,
         )
 
     def critical(
-        self, message=None, monitor=None, level=1, save=False, ex: Exception = None
+        self,
+        message=None,
+        monitor=None,
+        level=1,
+        log_in_db=False,
+        ex: Exception = None,
+        enabled: bool = True,
     ):
         self._log(
             message,
@@ -293,8 +361,9 @@ class AlphaLogger:
             stack_level=level,
             level="critical",
             monitor=monitor,
-            save=save,
+            log_in_db=log_in_db,
             ex=ex,
+            enabled=enabled,
         )
 
     def set_current_date(self):
@@ -336,23 +405,28 @@ class AlphaLogger:
     def trace_show(self):
         traceback.print_exc()
 
-    def __log_in_db(self, message, origin="unspecified", type_="unspecified", level=None):
+    def __log_in_db(
+        self, message, origin="unspecified", type_="unspecified", level: str = "info"
+    ):
         from ...models.database.main_definitions import Logs
+        from core import DB
 
         # Connect to db
-        stackraw = traceback.format_stack()
-        stack = "".join(stackraw) if stackraw is not None else ""
+        stack = ""
+        if type_ != "message":
+            stackraw = traceback.format_stack()
+            stack = "".join(stackraw) if stackraw is not None else ""
 
-        if self.database is not None:
-            self.database.insert(
-                Logs,
-                values={
-                    Logs.type_: type_,
-                    Logs.origin: origin,
-                    Logs.message: message,
-                    Logs.stack: stack,
-                },
-            )
+        DB.insert_or_update(
+            Logs,
+            values={
+                Logs.type_: type_,
+                Logs.origin: origin,
+                Logs.message: message,
+                Logs.stack: stack,
+                Logs.level: level,
+            },
+        )
 
     def print_error(self, error_msg, raise_exception=True):
         """Display the last error catched"""
@@ -367,20 +441,21 @@ class AlphaLogger:
 
     def process_log(self, uuid_process, name, parameters, status):
         from ...models.database.main_definitions import Processes
+        from core import DB
 
         if type(parameters) != str:
             parameters = ";".join([str(x) for x in parameters])
 
-        if self.database is not None:
-            self.database.insert(
-                Processes,
-                values={
-                    Processes.uuid: uuid_process,
-                    Processes.name: name,
-                    Processes.parameters: parameters,
-                    Processes.status: status,
-                },
-            )
+        DB.insert(
+            Processes,
+            values={
+                Processes.uuid: uuid_process,
+                Processes.name: name,
+                Processes.parameters: parameters,
+                Processes.status: status,
+            },
+        )
+
 
 """
 @singleton
