@@ -4,13 +4,17 @@ import os, sys, warnings
 from types import ModuleType
 from typing import List, Dict
 
+from sqlalchemy import MetaData
+
+from ...database.models import AlphaTable
+
 with warnings.catch_warnings():
     from flask_marshmallow import Marshmallow
 from flask_sqlalchemy import DefaultMeta
 
 from ....models.main import AlphaClass
 from ....models.logger import AlphaLogger, DEFAULT_FORMAT, DEFAUT_DATE_FORMAT
-from ....libs import flask_lib
+from ....libs import flask_lib, io_lib
 
 from ....models.config import AlphaConfig
 from ....models.main import AlphaException
@@ -23,6 +27,7 @@ from ....utils.tasks import start_celery
 
 from ....config.main_configuration import CONFIGURATION
 
+
 def _get_relative_path(file: str, level=0, add_to_path=True):
     if level == 0:
         root = os.path.dirname(file)
@@ -35,6 +40,7 @@ def _get_relative_path(file: str, level=0, add_to_path=True):
 
 class AlphaCore(AlphaClass):
     instance = None
+    tables = None
 
     def __init__(self, file: str, level: int = 0, *args, **kwargs):
         super().__init__(log=None)
@@ -52,7 +58,7 @@ class AlphaCore(AlphaClass):
         self._db: AlphaDatabase = None
         self.api: AlphaFlask = None
 
-        self.models_sources: List[str] = []
+        self._models_sources: List[str] = []
         self.databases_models: list = []
 
         configuration = (
@@ -68,12 +74,12 @@ class AlphaCore(AlphaClass):
             root=self.root,
             configuration=configuration,
             core=self,
-            #default=f"{self.root_alpha}{os.sep}{CONFIGURATION.DEFAULT_CONFIG_FILEPATH}.json"
+            # default=f"{self.root_alpha}{os.sep}{CONFIGURATION.DEFAULT_CONFIG_FILEPATH}.json"
         )
         """ignored_warnings = self.config.get("ignored_warnings", [])
         for ignored_warning in ignored_warnings:
             warnings.filterwarnings("ignore", message=ignored_warning)"""
-        
+
         self.logger_root = self.config.get("directories/logs", required=True)
         self.log: AlphaLogger = None
 
@@ -84,11 +90,9 @@ class AlphaCore(AlphaClass):
     def db(self):
         self.prepare_api(self.configuration)
         return self._db
-    
+
     def __set_loggers(self):
-        colors_loggers_default_file_path = (
-            f"{self.root_alpha}{os.sep}{CONFIGURATION.DEFAULT_LOGGERS_COLORS_FILEPATH}.json"
-        )
+        colors_loggers_default_file_path = f"{self.root_alpha}{os.sep}{CONFIGURATION.DEFAULT_LOGGERS_COLORS_FILEPATH}.json"
         loggers_default_colors_config = None
         if os.path.isfile(colors_loggers_default_file_path):
             loggers_default_colors_config = AlphaConfig(
@@ -150,7 +154,7 @@ class AlphaCore(AlphaClass):
             cmd_output=logger_config.get("cmd_output", default=True),
             level=logger_config.get("level"),
             colors=colors,
-            database=logger_config.get("database"),
+            log_in_db=logger_config.get("log_in_db"),
             excludes=logger_config.get("excludes"),
             config=logger_config.get("config"),
             replaces=logger_config.get("replaces"),
@@ -268,21 +272,33 @@ class AlphaCore(AlphaClass):
             elif db_type == "sqlite":
                 cnx_str = "sqlite:///" + cf_db["path"]
 
+            if "ssl" in cf_db and cf_db["ssl"] is not None:
+                if not all(
+                    elem in cf_db["ssl"].keys()
+                    for elem in ["ssl_ca", "ssl_cert", "ssl_key"]
+                ):
+                    raise AlphaException(
+                        description=f"'ssl_ca', 'ssl_cert' and 'ssl_key' are required in ssl configuration"
+                    )
+                ssl_ca = cf_db["ssl"]["ssl_ca"]
+                ssl_cert = cf_db["ssl"]["ssl_cert"]
+                ssl_key = cf_db["ssl"]["ssl_key"]
+
+                cnx_str = (
+                    f"{cnx_str}?ssl_ca={ssl_ca}&ssl_cert={ssl_cert}&ssl_key={ssl_key}"
+                )
+                if "ssl_check_hostname" in cf_db["ssl"].keys():
+                    cnx_str = f"{cnx_str}&ssl_check_hostname={cf_db['ssl']['ssl_check_hostname']}"
+
             if cnx_str is not None:
                 cf_db["cnx"] = cnx_str
-                db_cnx[db_name] = cf_db
+                db_cnx[db_name.upper()] = cf_db
 
         self.db_cnx = db_cnx
 
     def get_database(self, name):
         return self._db
 
-    def check_databases(self):
-        if self.configuration == "local":
-            self.prepare_api(self.configuration)
-            self.load_models_sources()
-            self._db.create_all()
-
     def set_configuration(self, configuration_name):
         if configuration_name is None and self.config.configuration is not None:
             configuration_name = self.config.configuration
@@ -297,7 +313,13 @@ class AlphaCore(AlphaClass):
 
         self.set_configuration(configuration)
 
-        template_path = os.path.dirname(__file__).split(os.sep+"alphaz"+os.sep)[0] + os.sep + 'alphaz' + os.sep + "templates"
+        template_path = (
+            os.path.dirname(__file__).split(os.sep + "alphaz" + os.sep)[0]
+            + os.sep
+            + "alphaz"
+            + os.sep
+            + "templates"
+        )
         self.api = AlphaFlask(
             __name__,
             template_folder=template_path,
@@ -308,33 +330,38 @@ class AlphaCore(AlphaClass):
             root=self.root,
             core=self,
         )
-        CORS(self.api, resources={r"*": {"origins": "*"}}, supports_credentials=True)
-        logging.getLogger('flask_cors').level = logging.DEBUG
-        self.api.config['CORS_HEADERS'] = 'Content-Type'
+        # if self.config.get("cors"):
+        # CORS(self.api, resources={r"*": {"origins": "*"}}, supports_credentials=True)
+        # logging.getLogger("flask_cors").level = logging.DEBUG
+        self.api.config["CORS_HEADERS"] = "Content-Type"
 
         self.ma = self.api.ma
 
         # Cnx
-        db_cnx = self.db_cnx
-        if db_cnx is None:
+        if self.db_cnx is None:
             self.error(
                 f"Databases are not configurated in config file {self.config.filepath}"
             )
             exit()
 
-        if not CONFIGURATION.MAIN_DATABASE_NAME in db_cnx:
+        if not CONFIGURATION.MAIN_DATABASE_NAME in self.db_cnx:
             self.config.show()
             self.config.error(
                 f"Missing <{CONFIGURATION.MAIN_DATABASE_NAME}> database configuration"
             )
             exit()
+        self.db_cnx = (
+            {x.upper(): y for x, y in self.db_cnx.items()}
+            if self.db_cnx is not None
+            else {}
+        )
 
         # bind = not "NO_BIND" in os.environ or not "Y" in str(os.environ["NO_BIND"]).upper()
         # if bind:
-        self.api.set_databases(db_cnx)
+        self.api.set_databases(self.db_cnx)
 
         # databases
-        db_logger = self.get_logger("database")
+        db_logger = self.get_logger("sqlalchemy.engine")
         if db_logger is None:
             db_logger = self.get_logger(CONFIGURATION.MAIN_LOGGER_NAME)
 
@@ -343,14 +370,12 @@ class AlphaCore(AlphaClass):
             name=CONFIGURATION.MAIN_DATABASE_NAME,
             main=True,
             log=db_logger,
-            config={"type": "oracle"},
+            config=self.config.get("orm", default={"type": "oracle"}),
         )  #!TODO: remove oracle type
-
-        # configuration
         self.api.db = self._db
 
-        """# ensure tests
-        self._db.ensure("tests", drop=True)
+        # ensure tests
+        """self._db.ensure("tests", drop=True)
         self._db.ensure("files_process")"""
 
     def get_database(self, name=None) -> AlphaDatabase:
@@ -368,60 +393,117 @@ class AlphaCore(AlphaClass):
 
     def get_table(self, schema: str, table: str):
         db = self.get_database(schema)
-        registered_classes = [
-            x for x in db.Model._decl_class_registry.values()
-        ]
-        registered_models: Dict[str, DefaultMeta] = {
-            x.__tablename__: x for x in registered_classes if isinstance(x, DefaultMeta)
-        }
+        return db.get_table(schema=schema, table=table)
 
-        if not table in registered_models:
-            raise AlphaException("cannot_find_table", parameters={"table": table})
-        return registered_models[table]
-
-    def create_table(self, schema: str, table_name: str):
+    def database_init_all(
+        self,
+        binds: List[str] = None,
+        tables: List[AlphaTable] = None,
+        drop: bool = False,
+        create: bool = False,
+        sqlite: bool = True,
+    ):
+        self.prepare_api(self.configuration)
         self.load_models_sources()
-        founds = []
 
-        for model in self.databases_models:
-            if not hasattr(model, "__tablename__"):
-                continue
-            if model.__tablename__.lower() == table_name.lower():
-                founds.append(model)
-        
-        """if len(founds) != 1:
-            founds = [
-                x
-                for x in founds
-                if not hasattr(model, "__bind_key__")
-                or (
-                    hasattr(model, "__bind_key__")
-                    and schema.lower() == model.__bind_key__.lower()
-                )
-            ]"""
-        if len(founds) == 1:
-            founds[0].create()
-            return True
-        return False
+        if sqlite:
+            binds = [
+                x.upper()
+                for x, y in self.db_cnx.items()
+                if y["type"] == "sqlite" and (binds is None or x in binds)
+            ]
 
-    def drop_table(self, schema: str, table_name: str):
+        """for bind, tables_dict in self.tables.items():
+            if binds is None or bind in binds:
+                for table_name, table_model in tables_dict.items():
+                    engine = self.db.get_engine(bind=bind)
+                    if drop:
+                        table_model.__table__.drop(engine)
+                    table_model.__table__.create(engine)"""
+        """if drop:
+            self._db.drop_all(bind=binds, tables=tables)
+        self._db.create_all(bind=binds, tables=tables)"""
+
+        tables_by_bind = {}
+        for table_model in tables:
+            bind = (
+                "MAIN"
+                if not hasattr(table_model, "__bind_key__")
+                else table_model.__bind_key__
+            )
+            if not bind in tables_by_bind:
+                tables_by_bind[bind] = [table_model]
+            else:
+                tables_by_bind[bind].append(table_model)
+
+        for bind, tables_models in tables_by_bind.items():
+            tables = [x.__table__ for x in tables_models]
+            meta = MetaData(self.db.get_engine(bind=bind))
+            if drop:
+                meta.drop_all(tables=tables)
+            meta.create_all(tables=tables)
+
+    def create_table(self, schema: str, table: str, drop: bool = False):
         db = self.get_database(schema)
-        table_object = self.get_table(schema, table_name)
+        table_object = self.get_table(schema, table)
+        if drop:
+            try:
+                table_object.__table__.drop(db.engine)
+            except:
+                pass
+        table_object.__table__.create(db.engine)
+        return True
+
+    def drop_table(self, schema: str, table: str):
+        db = self.get_database(schema)
+        table_object = self.get_table(schema, table)
         table_object.__table__.drop(db.engine)
 
+    @property
+    def models_sources(self):
+        if len(self._models_sources) == 0:
+            self._models_sources = self.config.get("directories/database_models")
+            if not self._models_sources:
+                self.log.error(
+                    f"Missing <directories/database_models> entry in configuration {self.conf.filepath}"
+                )
+                exit()
+        return self._models_sources
+
+    @models_sources.setter
+    def models_sources(self, models_sources):
+        self._models_sources = models_sources
+
     def load_models_sources(self):
-        if len(self.models_sources) != 0:
+        if len(self.models_sources) != 0 and len(self.databases_models) != 0:
             return
 
-        self.models_sources = self.config.get("directories/database_models")
-        if not self.models_sources:
-            self.log.error(
-                f"Missing <directories/database_models> entry in configuration {self.conf.filepath}"
-            )
-            exit()
-        
         self.log.info(f"Getting models definitions from {self.models_sources}")
-        self.databases_models = flask_lib.get_definitions_modules(self.models_sources, log=self.log)
+        (
+            self.databases_models,
+            self.tables,
+        ) = flask_lib.get_definitions_modules_and_tables(
+            self.models_sources, log=self.log
+        )
+
+    def get_table_model(self, tablename: str):
+        return flask_lib.get_table_model(tablename, self.models_sources, log=self.log)
 
     def get_data_directory(self):
         return self.config.get("directories/data", required=True)
+
+    def save(self, object_to_save: object, filename: str, ext: str = None):
+        filename = (
+            filename
+            if os.sep in filename
+            else self.config.get("directories/tmp") + os.sep + filename
+        )
+        io_lib.archive_object(object_to_save=object_to_save, filename=filename, ext=ext)
+
+    def load(self, filename: str, ext: str = None, default: object = None):
+        filename = (
+            filename
+            if os.sep in filename
+            else self.config.get("directories/tmp") + os.sep + filename
+        )
+        return io_lib.unarchive_object(filename=filename, ext=ext, default=default)
